% application 2: CARS93 data set
% This script processes the 93cars data set and compare the 
% non-iterativre algorithm with an iterative algorithmproduces

clear all

savepic=true;% save picture at the end
figurefilename='figure6.eps';
epsilon=1/10000;%jitter constant
% use seed to be able to duplicate this result
RandStream.setDefaultStream(RandStream('mt19937ar','seed',12345));

%%%%%%%%%%%%%%%%
% load data:
%%%%%%%%%%%%%%%%
load('93cars.mat')
% List of the loaded variables:
% Y: the matrix of 5 response variables
% XX: matrix of 14 explanatory variables

% now standardize X and y

Y_z=zscore(Y);
XX_z=zscore(XX);

% setting up y and X for application2, i.e., putting all y's along each
% other and making the block diangonal design matrix X.
y=[Y_z(:,1) ; Y_z(:,2) ;Y_z(:,3) ;Y_z(:,4) ;Y_z(:,5)];
X=blkdiag(XX_z,XX_z,XX_z,XX_z,XX_z);



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Group quantile regression %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% add some small amount of random jitter to the data:
numcol=size(X,2);
for m=1:numcol
    jitter=randn(size(X(:,m)))*(std(X(:,m))*epsilon);
    X(:,m)=X(:,m)+jitter;
end
y=y+randn(size(y))*std(y)*epsilon;

% define groups:
numgroups=14;
numvar=5;
for i=1:numgroups
    groups{i}=i:numgroups:numgroups*numvar;
    disp(['Group ' num2str(i) ': ' num2str(groups{i})]);
end

% set quantile and stopping condition of algorithm
tau=0.5;
stopping.lambda=0;
% NB: qrmix computes minimizers of ||A*x-b||_1 instead of ||y-X*beta|\_1
% (sign change = left/right change)
weights.left=(2*tau)*ones(size(X,1),1);
weights.right=(2*(1-tau))*ones(size(X,1),1);
includetest=true;
verbose=0;
% compute solution path
disp('Start of qrmix...');
tic;
[xlist,lambdalist,u0list,u1list,w0list,w1list,succes]=qrmix(X,y,groups,stopping,verbose,weights,includetest);
timespent=toc;
disp('End of qrmix');
disp(['Time spent: ' num2str(timespent)]);

% compute number of break points returned by qrmix
numpts=size(xlist,2);

% calculate mixed norm R for break point in xlist
Rlist=zeros(1,numpts);
for i=1:numpts
    Rlist(i)=mixednorm(xlist(:,i),groups);
end

% Compute the  \ell_{\infty} norm of \hat{\beta}'s of each group:
infnorms=zeros(numgroups,numpts);
for i=1:numgroups
    for j=1:numpts
        infnorms(i,j)=norm(xlist(groups{i},j),inf);
    end
end

% compute loss (only correct for tau=0.5)
for i=1:numel(Rlist) loss1(i)=norm(X*xlist(:,i)-y,1);end;


% test iterative algorithm
numit=100;
xstart=zeros(size(X,2),1);
vstart=zeros(size(X,1),1);
left=weights.left;
right=weights.right;
stepsize=0.99/svds(X,1);%<------
iterativeminimizer=zeros(size(xlist));
iterativedist=zeros(numel(Rlist),numit);
finaliterativeloss=zeros(1,numel(Rlist));
tic;
for i=1:numel(Rlist)
    if mod(i,100)==1 disp(num2str(i));end;
R=Rlist(i);
xref=xlist(:,i);
iterativeresult=iterativegroupqr(X,y,left,right,R,groups,numit,xstart,vstart,xref,stepsize);

finaliterativeloss(i)=iterativeresult.losslist(end);
%iterativemixednorm=iterativeresult.groupnormlist;
iterativeminimizer(:,i)=iterativeresult.minimizer;
iterativedist(i,:)=iterativeresult.dist;
end;
timespent=toc;
disp(['Time spent (iterative): ' num2str(timespent)]);
disp(['Time spent per run: ' num2str(timespent/numel(Rlist)) 's. (' num2str(numel(Rlist)) ' runs).']);



%%%%%%%%%%%%%%%
% compute loss  (only correct for tau=0.5)
for i=1:numel(Rlist) loss2(i)=norm(X*iterativeminimizer(:,i)-y,1);end;




%%%%%%%%%%%%%%%%%%%%%%%%%%%
% plot iteration and trade-off curve
sample=600;%take this R from Rlist
fontsize=10;
figure;

% iteration
subplot(1,2,1)
plot(iterativedist(sample,:),'-k');
axis([1 numit 0 1]);
set(gca,'YTick',[0 0.2 0.4 0.6 0.8 1]);
set(gca,'XTick',[20 40 60 80 100]);
% horizontal label
text(numit/2, -0.15,'n','HorizontalAlignment','center','FontSize',fontsize)
% vertical label
text('Interpreter','latex','HorizontalAlignment','center',...
    'FontName','Helvetica',...
    'String','$\|\beta^{(n)}-\hat\beta\|_2/\|\hat\beta\|_2$',...
    'Position',[-14 0.5],...
    'Rotation',90,...
    'FontSize',fontsize)
% text in figure
text(75, 0.9,['R=' num2str(Rlist(sample))],'FontSize',fontsize)
pbaspect([2 1 1])



% final error for all R
subplot(1,2,2)
plot(Rlist,iterativedist(:,end),'-k');
axis([0 max(Rlist) 0 1]);

set(gca,'YTick',[0 0.2 0.4 0.6 0.8 1]);
set(gca,'XTick',[0 1 2 3 4 5]);
%xlabel('R');
%ylabel('loss');
%title('Final error (as a function of R)');
% horizontal label
text(1.02*(Rlist(1)+Rlist(end))/2, -0.15,'R','HorizontalAlignment','center','FontSize',fontsize)
% vertical label
text('Interpreter','latex','HorizontalAlignment','center',...
    'FontName','Helvetica',...
    'String','$\|\beta^{(n)}-\hat\beta\|_2/\|\hat\beta\|_2$',...
    'Position',[-0.7 0.5],...
    'Rotation',90,...
    'FontSize',fontsize)
% text in figure
text(0.9*Rlist(end), 0.9,['n=' num2str(numit)],'HorizontalAlignment','center', 'FontSize',fontsize)
pbaspect([2 1 1])
    

    
if savepic saveas(gcf,figurefilename);end;
    
disp(['Number of break points: ' num2str(numel(Rlist))]);
disp(['Index of break point chosen in figure: ' num2str(sample)]);
disp(['Mixed norm of break point chosen: ' num2str(Rlist(sample))]);
disp('Done.');

