function [mu1,mu2]=findl1ballintersection(a,b,radius)
% find mu such that ||a+mu*b||_1=radius

% Last modified by Ignace Loris (igloris@ulb.ac.be) on 22/10/2012

if norm(b)==0
    mu1=NaN;
    mu2=NaN;
else
    a=reshape(a,1,[]);
    b=reshape(b,1,[]);
    % find knots  and add two values that are certain to bring you outside ||x||_1=radius
    d=dot(a,b)^2-norm(b)^2*(norm(a)^2-1.1*radius^2);
    if d>0
        tmp1=(-dot(a,b)+sqrt(d))/norm(b)^2;
        tmp2=(-dot(a,b)-sqrt(d))/norm(b)^2;
    else
        tmp1=0;
        tmp2=1;
    end
    mulist=sort([reshape(-a./b,1,[]), tmp1, tmp2],'ascend');
    % calculate value of ||a+mu*b||_1 at those knots
    values=zeros(1,numel(mulist));
    for i=1:numel(values)
        values(i)=norm(a+mulist(i)*b,1);
    end
    if min(values)>radius
        mu1=NaN;
        mu2=NaN;
    else
        % first intersection point (smallest mu)
        for i=1:numel(values)-1
            if values(i)>=radius && values(i+1)<=radius
                mua1=mulist(i);
                mua2=mulist(i+1);
                v1=values(i);
                v2=values(i+1);
                %linear interpolation
                mu1=mua1+(mua2-mua1)/(v2-v1)*(radius-v1);
                break;
            end
        end
        % second intersection point (largest mu)
        for i=numel(values):-1:2
            if values(i)>=radius && values(i-1)<=radius
                mua1=mulist(i);
                mua2=mulist(i-1);
                v1=values(i);
                v2=values(i-1);
                %linear interpolation
                mu2=mua1+(mua2-mua1)/(v2-v1)*(radius-v1);
                break;
            end
        end
    end
end