function f=iterativegroupqr(K,y,left,right,R,groups,numit,xstart,vstart,xref,stepsize)
% use an iterative algorithm to compute the minimizer of 
% \|Kx-y\|_1 with ||x||_mix<=R
% or more generally with left/right weights on the 1-norm (both positive)
% output is a structure with the minimizer beta, a list of values of the
% loss, a list of values of the group norm of beta

% Last modified by Ignace Loris (igloris@ulb.ac.be) on 15/07/2013

x=xstart;
v=vstart;
vp=v;
xrefnrm=norm(xref);if xrefnrm==0 xrefnorm=1;end;

losslist=zeros(1,numit);
groupnormlist=zeros(1,numit);
dist=zeros(1,numit);
for i=1:numit
    x=projmixednormball(x-stepsize*K'*(2*v-vp),groups,R);
    groupnormlist(i)=mixednorm(x,groups);
    dist(i)=norm(x-xref)/xrefnrm;
    vp=v;
    tmp=K*x-y;
    
    ind1=tmp>=0;
    ind2=tmp<0;
    losslist(i)=sum(right(ind1).*tmp(ind1))-sum(left(ind2).*tmp(ind2));
    
    v=projasyminf(v+stepsize*tmp,-left,right);
end

f.minimizer=x;
f.losslist=losslist;
f.groupnormlist=groupnormlist;
f.dist=dist;